/*file*********** MELNK_DRV : ***********************************************/
/*                                                                          */
/*                                                                          */
/*                JL080INI.C : JL-080 interface access driver               */
/*                                                                          */
/*                                                                          */
/******** Copyright(C) 2002 Yaskawa Electric Corporation ********************/
/*                                                                          */
/*      Ver 1.00    2002.03.01   Initial coding    I.Matsumura              */
/*      Ver 1.10    2002.06.01                     I.Matsumura              */
/*                - add memory check function. :yns_check_ram()             */
/*                - modify the global data code.                            */
/*      Ver 1.11    2003.02.25                     I.Matsumura              */
/*                - fix CTRL code of global frame.                          */
/*                - add max slots check.                                    */
/*      Ver 1.30    2003.11.14                     I.Matsumura              */
/*                <1> fix calculation of ubp reg. data                      */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*                                                                          */
/*       [Functions]                                                        */
/*       Service functions                                                  */
/*       yns_init_channel()          Initialize Channel Buffer              */
/*       yns_reset()                 Reset JL-080                           */
/*       yns_set_regs()              Initialize Host I/F Registers          */
/*       yns_set_param()             Set Communication Parameter            */
/*       yns_enable()                Enable JL-080                          */
/*       yns_sequent_init()          Execute JL-080 initializing sequense   */
/*       yns_check_ram()             memory check                           */
/*                                                                          */
/*                                                                          */
/******** Copyright(C) 2002 Yaskawa Electric Corporation ********************/

#ifndef YENET_LOCAL_DEFINES_
#define YENET_LOCAL_DEFINES_
#endif /* _YENET_LOCAL_DEFINES */

/* includes */
#include "jl080gbl.h"

/* local defines */
#define	YN_MOD_INVALID	0x0200	/* BMOD invalid code */
#define YN_PROSEL_CHECK	0x0006	/* Protocol check */

#define	YENET_POLLING_READY	0x0ffff	/* Polling counter for yenet_reset */
#define	YENET_POLLING_COUNT	0x0ffff	/* Polling counter for service func. */

#define	YN_GBL_CODE		0x0800	/* Global code for C1 master */
#define	YN_MGBL_CODE	0x0300	/* Global code for C1 master */
#define	YN_SGBL_CODE	0x0100	/* Global code for slave */

/* forward declarations */
#ifdef __cplusplus
extern	"C"	{
#endif
void yns_init_channel( CHANNEL_INFO *chbuffp, USHORT *hif_reg_top );
int yns_reset( CHANNEL_INFO *chbuffp );
int yns_set_regs( CHANNEL_INFO *chbuffp, YENET_REG_PAR *regp );
int yns_set_param( CHANNEL_INFO *chbuffp, YENET_COM_PAR *par );
int yns_enable( CHANNEL_INFO *chbuffp );
void yns_set_transmit_data( CHANNEL_INFO *chbuffp,
				YENET_REG_PAR *reg_par, YENET_COM_PAR *com_par, UCHAR *adrp);
void yns_set_lnk_header( CHANNEL_INFO *chbuffp, 
						UINT st_no, USHORT bmod, USHORT ctrl);
int yns_sequent_init( CHANNEL_INFO *chbuffp, USHORT *hif_reg_top,
					 YENET_USER_PAR *usr_par, UCHAR *adrsp );
int yn_set_user_prm( YENET_USER_PAR *par, 
				YENET_REG_PAR *reg_par, YENET_COM_PAR *com_par );
int yns_check_ram( USHORT *top_adr, 
						UINT ofst_adr, UINT size, USHORT chk_data);
int yns_reset_prm( CHANNEL_INFO *chbuffp, USHORT t_mcyc );
#ifdef __cplusplus
}
#endif


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_init_channel()                Initialiize Channel buffer        */
/*                                                                          */
/*Function :                                                                */
/*      Clear Chnannel buffer and save host interface top address           */
/*                                                                          */
/*Reference :                                                               */
/*      set_user_prm                                                        */
/*                                                                          */
/*Return code :                                                             */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :   Channel buffer address                    */
/*      USHORT *hif_reg_top   :   Host I/F register top address             */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
void yns_init_channel( CHANNEL_INFO *chbuffp, USHORT *hif_reg_top )
{
	USHORT	*p;
	USHORT	i, size;

/* Clear channel buffer */
	p = (USHORT *)chbuffp;
	size = sizeof(CHANNEL_INFO) / sizeof(USHORT);
	for (i = 0; i < size; i++)
	{
		*p++ = 0;
	}

/* Set top address of Host interface registers */
	chbuffp->hif_reg_top = (YENET_HOST_IF_REGS *)hif_reg_top;

/* Set check code */
	chbuffp->check_code = YENET_INIT_CHBUF;

}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_reset()                       Reset JL-080                      */
/*                                                                          */
/*Function :                                                                */
/*      Reset JL-080 and wait JL-080 ready                                  */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                            Normal End                          */
/*      ERROR_INIT_CHANNEL :            Invalid channel buffer specified    */
/*      ERROR_NOT_READY :               CERF Not Ready (STS(RDY) is zero.)  */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :	        Channel buffer address              */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yns_reset( CHANNEL_INFO *chbuffp )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT sts;							/* STS reg. */
	USHORT i;							/* Polling Conter */
	int ret;							/* Return code */

/* Check Channel Buffer */
	if (chbuffp->check_code != YENET_INIT_CHBUF)
	{
		return ERROR_INIT_CHANNEL;
	}

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Reset CERF */
	ynb_set_cmd( hirp, YN_CMD_RST_CER );

/* Wait CERF Ready */
	ret = ERROR_NOT_READY;
	for (i = 0; i < YENET_POLLING_READY; i++)
	{
		sts = ynb_get_sts( hirp );
		if ((sts & YN_STS_RDY) != 0)
		{
			ret = OK;
			break;			/* Exit */
		}
	}

	return ret;
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yn_set_user_prm()              Set user parameter                   */
/*                                                                          */
/*Function :                                                                */
/*      Set specified parameter to host interface register and              */
/*      communication parameter register.                                   */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                            Normal End                          */
/*      ERROR_INVALID_PROTOCOL :        Unsupported protocol                */
/*      ERROR_INVALID_BMODE :           Unsupported buffer mode             */
/*      ERROR_RAM_SIZE_OVER :           using memory size larger than RAM   */
/*                                                                          */
/*Parameter :                                                               */
/*      YENET_USER_PAR *usr_par :       Communicaiotn parameter             */
/*      YENET_REG_PAR *reg_par :        Register parameter                  */
/*      YENET_COM_PAR *com_par :        Communicaiotn parameter             */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yn_set_user_prm( YENET_USER_PAR *par, 
				YENET_REG_PAR *reg_par, YENET_COM_PAR *com_par )
{
	USHORT	calc;

/* Check parameter */
	if(par->mod & YN_PROSEL_CHECK)
		return ERROR_INVALID_PROTOCOL;	/* protocol error */

	if((par->mod & YN_MOD_MASK) == YN_MOD_INVALID)
		return ERROR_INVALID_BMODE;	/* buffer mode error */

/* Set 1NS, 2NS Common parameter */
	reg_par->mod = 
		(par->mod & ~(YN_MOD_LBTST | YN_MOD_MTYP_C2)) | YN_MOD_CODE_MAN;
	reg_par->imk = DEF_IMK_REG;			/* IMK register */

	com_par->ma = par->ma;				/* My Address */
	com_par->ma_max = par->ma_max;		/* Max. Number of Slaves */
	com_par->t_mcyc = par->t_mcyc;		/* Tmcyc */
	com_par->t_cyc = par->t_cyc;		/* Tcyc */
	com_par->byte = par->byte;			/* Transimit Data Size (Bytes) */
	com_par->dev = par->dev;			/* Devision */
	com_par->prel = DEF_2NS_PREL;		/* Preamble Length */
	com_par->int1_dly = DEF_2NS_INT1_DLY;/* int1 Delay */
	com_par->ram_spd = DEF_2NS_RAM_SPD;	/* External RAM Speed */
	com_par->rom_spd = DEF_2NS_ROM_SPD;	/* External ROM Speed */

/* Set 1NS, 2NS parameter */
	if((par->mod & YN_MOD_PROCEL_2NS) != 0)	/* 2 : N */
	{
		reg_par->mctr = 						/* MCTR reg. */
				(com_par->byte/sizeof(USHORT)+1)*(par->ma_max+1)*2;
		reg_par->rmbd = ((reg_par->mod & YN_MOD_MASK) == 0) ? 
					reg_par->mctr + 1 : reg_par->mctr * 2;	/* RMBD reg. */
		com_par->lbss = reg_par->mctr;			/* Alt buf.B start Adrress*/
		com_par->max_rtry = par->max_rtry;		/* Max Number of Retry */
		com_par->c2m_ch = par->c2m_ch;			/* Time slot */
		com_par->msg_ptr = reg_par->rmbd + 32;	/* Message buf. pointer */

	/* calc total size */
		calc = com_par->msg_ptr + 
				DEF_2NS_C1MSG_SZ*2 + DEF_2NS_C1MSG_SZ*2;

		if(calc > DEF_RAM_SIZE)
			return ERROR_RAM_SIZE_OVER;			/* RAM size over */

		if((reg_par->mod & YN_MOD_TYPE_MASTER) != 0)	/* Master */
		{
			calc = 44 + com_par->t_cyc *
				(com_par->max_rtry + com_par->ma_max + com_par->c2m_ch + 1);

			if(calc > com_par->t_mcyc)
				return ERROR_SLOTCH_OVER;			/* SLOT CH over */
		}

		com_par->c1msg_sz = DEF_2NS_C1MSG_SZ;	/* C1 message size */
		com_par->c2msg_sz = DEF_2NS_C2MSG_SZ;	/* C2 message size */
		com_par->wdt = par->wdt;				/* Watch dog setting */
		com_par->c2dly = DEF_2NS_C2DLY;			/* Transfer C2-msg delay */
	}
	else	/* 1 : N */
	{
		reg_par->mctr = DEF_1NS_LBSS_REG;	/* Alt buf.B start Adrress*/
		reg_par->rmbd = DEF_1NS_RMBD_REG;	/* RMBD register */
		com_par->lbss = reg_par->mctr;
		com_par->max_rtry = 0;		/* Max Number of Retry */
		com_par->c2m_ch = 0;		/* Time slot */
		com_par->msg_ptr = 0;		/* Message buf. pointer */
		com_par->c1msg_sz = 0;		/* C1 message size */
		com_par->c2msg_sz = 0;		/* C2 message size */
		com_par->wdt = 0;			/* Watch dog setting */
		com_par->c2dly = 0;			/* Transfer message delay for C2 */
	}

	return OK;
}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_set_regs()                  Initialize Host I/F Registers       */
/*                                                                          */
/*Function :                                                                */
/*      Set specified data to host interface register and set request of    */
/*      resetting pointer.                                                  */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                            Normal End                          */
/*      ERROR_INIT_CHANNEL :            Invalid channel buffer specified    */
/*      ERROR_INVALID_PROTOCOL :        Unsupported protocol                */
/*      ERROR_INVALID_BMODE :           Unsupported buffer mode             */
/*      ERROR_POLL_TIMEOUT_REG :        P_RSTAK Request did not completed   */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :         Channel buffer address              */
/*      YENET_REG_PAR *par :            Register parameter                  */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yns_set_regs( CHANNEL_INFO *chbuffp, YENET_REG_PAR *regp )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT data;						/* Register data */
	int ret;							/* Return code */
	USHORT i;							/* Polling Conter */

/* Check Channel Buffer */
	if (chbuffp->check_code != YENET_INIT_CHBUF)
	{
		return ERROR_INIT_CHANNEL;
	}

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

	if(regp->mod & YN_PROSEL_CHECK)
	{
		return ERROR_INVALID_PROTOCOL;
	}
	if((regp->mod & YN_MOD_MASK) == YN_MOD_INVALID)
	{
		return ERROR_INVALID_BMODE;
	}

/* Save Buffer mode and rmbd */
	chbuffp->buff_mod = regp->mod & YN_MOD_MASK;
	chbuffp->pars_data = regp->rmbd;

/* Set Register */
	ynb_set_imk( hirp, regp->imk );	/* Interrupt Mask */
	ynb_set_mod( hirp, regp->mod );	/* Mod Register */
	ynb_set_pars( hirp, regp->rmbd );	/* Comm. Par. address */
	ynb_set_lbss( hirp, regp->mctr );

/* Write  Pointer Reset Request */
	ynb_set_cmd( hirp, YN_CMD_P_RSTRQ );

/* Wait Request End */
	ret = ERROR_POLL_TIMEOUT_REG;
	for (i = 0; i < YENET_POLLING_COUNT; i++)
	{
		data = ynb_get_ints( hirp );
		if ((data & YN_INTS_P_RSTAK) != 0)
		{
		/* Clear Interrupt Status */
			ynb_clear_ints( hirp, YN_INTS_P_RSTAK );
			ret = OK;
			break;				/* Exit */
		}
	}

	return ret;
}


/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_set_param()                 Set communication parameter         */
/*                                                                          */
/*Function :                                                                */
/*      Write communication parameter to common memory and set request of   */
/*      setting parameter.                                                  */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                              Normal End                        */
/*      ERROR_INIT_CHANNEL :              Invalid channel buffer specified  */
/*      ERROR_POLL_TIMEOUT_PAR :          PAR_I Request did not completed   */
/*      ERROR_INVALID_PARAMETER :         Invalid parameter                 */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :           Channel buffer address            */
/*      YENET_COM_PAR *par :              Communicaiotn parameter           */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*            1.30<1> fix calculation of ubp reg. data                      */
/*                                                                          */
/****************************************************************************/
int yns_set_param( CHANNEL_INFO *chbuffp, YENET_COM_PAR *par )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT data = 0;					/* Register data */
	USHORT *pp;							/* Com. Param Pointer */
	USHORT i;							/* Polling Conter */
	UINT par_sz;						/* Parameter size */
	int ret;							/* Return code */

/* Check Channel Buffer */
	if (chbuffp->check_code != YENET_INIT_CHBUF)
	{
		return ERROR_INIT_CHANNEL;
	}

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Save data to channel buffer */
	chbuffp->ma = par->ma;		/* My address */
	chbuffp->ma_max = par->ma_max;	/* Max. number of stations */
	chbuffp->link_sz = (par->byte >> 1)+1;	/* Buffer size */
	chbuffp->link_radr[0] = (par->lbss >> 1);
	chbuffp->link_radr[1] = par->lbss + (par->lbss >> 1);
	chbuffp->link_sadr[0] = 0;
	chbuffp->link_sadr[1] = par->lbss;
	chbuffp->c1msg_sz = par->c1msg_sz;
	chbuffp->c1msg_adr = par->msg_ptr;
	chbuffp->c2msg_sz = par->c2msg_sz;
	chbuffp->c2msg_adr = par->msg_ptr + par->c1msg_sz*2;
	chbuffp->msgpkt_sz = ((par->byte-1) >> 1);
	par_sz = sizeof(YENET_COM_PAR) >> 1; /* Set parameter size */

/* Write Parameter */
	pp = (USHORT*)par;
	ynb_set_ubp( hirp, chbuffp->pars_data);	/* Set RMBD start address */
	for(i = 0 ; i < par_sz ; i++)
	{
		ynb_set_ubd( hirp, *pp);
        pp++;
	}

/* Write  Com. Par. Request */
	ynb_set_cmd( hirp, YN_CMD_PAR_R );

/* Wait Request End */
	ret = ERROR_POLL_TIMEOUT_PAR;
	for (i = 0; i < YENET_POLLING_COUNT; i++)
	{
		data = ynb_get_ints( hirp );
		if ((data & YN_INTS_PAR_I) != 0)
		{
		/* Clear Interrupt Status */
			ynb_clear_ints( hirp, YN_INTS_PAR_I );

		/* Check Error Status */
			data = ynb_get_ests( hirp );
			if ((data & YN_ESTS_PAR_ERR) != 0)
			{
			/* Clear Error Status */
				ynb_clear_ests( hirp, YN_ESTS_PAR_ERR );
				ret = ERROR_INVALID_PARAMETER;
			}
			else
			{
				ret = OK;
			}
			break;
		}
	}

	return ret;
}


/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_enable()                    Enable JL-080                       */
/*                                                                          */
/*Function :                                                                */
/*      Enable JL-080 to transmit data                                      */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                              Normal End                        */
/*      ERROR_INIT_CHANNEL :              Invalid channel buffer specified  */
/*      ERROR_POLL_TIMEOUT_ENA :          Request did not completed         */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :           Channel buffer address            */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yns_enable( CHANNEL_INFO *chbuffp )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT data;						/* Register data */
	int ret;							/* Return code */
	USHORT i;							/* Polling Conter */

/* Check Channel Buffer */
	if (chbuffp->check_code != YENET_INIT_CHBUF)
	{
		return ERROR_INIT_CHANNEL;
	}

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Write  Enable Request */
	ynb_set_cmd( hirp, YN_CMD_ENA_CER );

/* Wait Request End */
	ret = ERROR_POLL_TIMEOUT_ENA;
	for (i = 0; i < YENET_POLLING_COUNT; i++)
	{
		data = ynb_get_cmd( hirp );
		if ((data & YN_CMD_ENA_CER) == 0)
		{
			ret = OK;
			break;
		}
	}

	return ret;
}


/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*     yns_set_transmit_data()             Set 1st transmit data            */
/*                                                                          */
/*Function :                                                                */
/*      Enable JL-080 to transmit data                                      */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :         Channel buffer address              */
/*      YENET_REG_PAR *reg_par :        Register parameter                  */
/*      YENET_COM_PAR *com_par :        Communicaiotn parameter             */
/*      UCHAR        *adrp    :         Station address                     */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
void yns_set_transmit_data( CHANNEL_INFO *chbuffp,
				YENET_REG_PAR *reg_par, YENET_COM_PAR *com_par, UCHAR *adrp)
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT data = 0;	/* Register data */
	int i;				/* general */

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

	if((reg_par->mod & YN_MOD_TYPE_MASTER) != 0)	/* Master */
	{
		for(i = 1; i <= chbuffp->ma_max; i++)
		{
			data = YN_MGBL_CODE | adrp[i];
		/* Set link header to Alt Buffer A */
			yns_set_lnk_header(chbuffp, i, 0, data);
		/* Set link header to Alt Buffer B */
			if(chbuffp->buff_mod == YN_MOD_BMOD_ALT)
			{
				yns_set_lnk_header(chbuffp, i, 1, data);
			}
		}
	/* Set global data to Alt Buffer A */
		data = ((reg_par->mod & YN_MOD_PROCEL_2NS) != 0)
				? (YN_GBL_CODE | adrp[0])	/* 2 : N */
				: adrp[0];					/* 1 : N */

		ynb_set_ubp(hirp, 0);
		ynb_set_ubd(hirp, data);
		ynb_set_ubd(hirp, com_par->t_mcyc);
		ynb_set_ubd(hirp, com_par->t_cyc);
		ynb_set_ubd(hirp, com_par->c2m_ch);

	/* Set global data to Alt Buffer B */
		if(chbuffp->buff_mod == YN_MOD_BMOD_ALT)
		{
			ynb_set_ubp(hirp, com_par->lbss);
			ynb_set_ubd(hirp, data);
			ynb_set_ubd(hirp, com_par->t_mcyc);
			ynb_set_ubd(hirp, com_par->t_cyc);
			ynb_set_ubd(hirp, com_par->c2m_ch);
		}
	}
	else	/* Slave */
	{
		data = YN_SGBL_CODE | chbuffp->ma;
	/* Set link header to Alt Buffer A */
		yns_set_lnk_header(chbuffp, 0, 0, data);
	/* Set link header to Alt Buffer B */
		if(chbuffp->buff_mod == YN_MOD_BMOD_ALT)
		{
			yns_set_lnk_header(chbuffp, 0, 1, data);
		}
	}
}


/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_set_lnk_header()             Set link header code               */
/*                                                                          */
/*Function :                                                                */
/*      Set link header code                                                */
/*                                                                          */
/*Return code :                                                             */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :           Channel buffer address            */
/*      UINT         st_no    :           Statin number                     */
/*      USHORT       bmod     :           ALT Buf.  A or B                  */
/*                                        (bmod=0: A  bmod!=0: B)           */
/*      USHORT       ctrl     :           link header (St.Adrs+CTRL)        */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*            1.30<1> fix calculation of ubp reg. data                      */
/*                                                                          */
/****************************************************************************/
void yns_set_lnk_header( CHANNEL_INFO *chbuffp, 
						UINT st_no, USHORT bmod, USHORT ctrl)
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT ubp_data;					/* Register data */

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;

/* Get receive buffer offset */
	ubp_data = 0;
	if (bmod != 0)
	{
		ubp_data += chbuffp->link_sadr[1];
	}

/* Add offset of station number */
	ubp_data += (USHORT)(chbuffp->link_sz * st_no);

/* Set offset address to UBP.reg */
	ynb_set_ubp( hirp, ubp_data );

/* Write UBD data */
	ynb_set_ubd( hirp, ctrl );

}


/*func*/
/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_sequent_init()                Initialize Host I/F Registers	    */
/*                                                                          */
/*Function :                                                                */
/*      Execute initializing sequense of JL-080.                            */
/*      Initializing sequense is following.                                 */
/*                                                                          */
/*      1. Reset JL-080 (yns_reset).                                        */
/*      2. Set specified data to host interface register (yns_set_regs).    */
/*      3. Set Communication parameter (yns_set_param).                     */
/*      4. Enable CERF (yns_enable).                                        */
/*      5. Set 1st Transmit data.                                           */
/*      6. Request Link data Transfer.                                      */
/*                                                                          */
/*Return code :                                                             */
/*      OK :                              Normal End                        */
/*      ERROR_INIT_CHANNEL :              Invalid channel buffer specified  */
/*      ERROR_POLL_TIMEOUT :              Request did not completed         */
/*      ERROR_NOT_READY :                 CERF Not Ready (STS(RDY) is zero.)*/
/*      ERROR_INVALID_PROTOCOL :          Unsupported protocol              */
/*      ERROR_INVALID_BMODE :             Unsupported buffer mode           */
/*      ERROR_POLL_TIMEOUT_REG :          P_RSTAK Request did not completed */
/*      ERROR_POLL_TIMEOUT_PAR :          PAR_I Request did not completed   */
/*      ERROR_POLL_TIMEOUT_ENA :          Request did not completed         */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :           Channel buffer address            */
/*      USHORT *hif_reg_top   :           Host I/F register top address     */
/*      YENET_USER_PAR *usr_par :         Communicaiotn parameter           */
/*      UCHAR *adrsp          :           Station address                   */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yns_sequent_init( CHANNEL_INFO *chbuffp, USHORT *hif_reg_top,
					 YENET_USER_PAR *usr_par, UCHAR *adrsp )
{
	YENET_COM_PAR par;	/* Communication parameter */
	YENET_REG_PAR reg;	/* Register parameter */
	int ret = OK;		/* Return code */

/* Initialize channel buffer */
	yns_init_channel(chbuffp,(USHORT*)hif_reg_top);

/* Set default parameter */
	if((ret = yn_set_user_prm( usr_par, &reg, &par )) != OK)
	{
		return ret;
	}

/* Reset CERF */
	if ((ret = yns_reset(chbuffp)) != OK)
	{
	/* Error end */
		return ret;
	}

/* Set Registers */
	if ((ret = yns_set_regs(chbuffp, &reg)) != OK)
	{
	/* Error end */
		return ret;
	}

/* Set Communication Parameter */
	if ((ret = yns_set_param(chbuffp, &par)) != OK)
	{
	/* Error end */
		return ret;
	}

/* Enable CERF */
	if ((ret = yns_enable(chbuffp)) != OK)
	{
	/* Error end */
		return ret;
	}

/* Set 1st Transmit data */
	yns_set_transmit_data(chbuffp, &reg, &par, adrsp);

/* Request Transmit flag */
	ynb_set_cmd(chbuffp->hif_reg_top, YN_CMD_LTX_RA);

	return ret;
}

/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_check_ram()                   RAM Check                         */
/*                                                                          */
/*Function :                                                                */
/*                                                                          */
/*Return code :                                                             */
/*      ERROR_RAM_CHECK       :           RAM Check error                   */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :           Channel buffer address            */
/*      UINT         ofst_adr :           Ofset address of memory area      */
/*      UINT         size     :           memory check size                 */
/*      USHORT       chk_data :           check data(write data)            */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*                                                                          */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yns_check_ram( USHORT *hif_reg_top, 
						UINT ofst_adr, UINT size, USHORT chk_data)
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	UINT chk_size;						/* Check Data Area size */
	USHORT rd_data;						/* Read data */
	int ret, i;							/* return code */

/* Get Host I/F register top address */
	hirp = (volatile YENET_HOST_IF_REGS*)hif_reg_top;

/* Reset CERF */
	ynb_set_cmd( hirp, YN_CMD_RST_CER );

/* Wait CERF Ready */
	ret = ERROR_NOT_READY;
	for (i = 0; i < YENET_POLLING_READY; i++)
	{
		rd_data = ynb_get_sts( hirp );
		if ((rd_data & YN_STS_RDY) != 0)
		{
			ret = OK;
			break;			/* Exit */
		}
	}
	if(ret != OK)	return ret;	/* Can't reset jl-080 */

/* Write UBD data */
	chk_size = size;
	ynb_set_ubp( hirp, ofst_adr );
	while(chk_size > 0)
	{
		ynb_set_ubd( hirp, chk_data );
		chk_size--;
	}

/* Read UBD data, and verify writed data */
	chk_size = size;
	ynb_set_ubp( hirp, ofst_adr );
	while(chk_size > 0)
	{
		rd_data = ynb_get_ubd( hirp );
		if(rd_data != chk_data)
			return ERROR_RAM_CHECK;
		chk_size--;
	}

	return OK;
}


/****************************************************************************/
/*                                                                          */
/*Name :                                                                    */
/*      yns_reset_param()               Reset communication parameter       */
/*                                                                          */
/*Function :                                                                */
/*      Write communication parameter to common memory and set request of   */
/*      setting parameter.                                                  */
/*                                                                          */
/*Reference :                                                               */
/*                                                                          */
/*Return code :                                                             */
/*                                                                          */
/*Parameter :                                                               */
/*      CHANNEL_INFO *chbuffp :           Channel buffer address            */
/*      t_mcyc                :           Transmission cycle time           */
/*                                                                          */
/*Reused module :                                                           */
/*                                                                          */
/*Note :                                                                    */
/*    Slave station can only use this function                              */
/*Log :                                                                     */
/*                                                                          */
/****************************************************************************/
int yns_reset_prm( CHANNEL_INFO *chbuffp, USHORT t_mcyc )
{
	volatile YENET_HOST_IF_REGS *hirp;	/* Host I/F Top address */
	USHORT	data;			/* Register data */
	USHORT	ubp_back;
	int		ret = OK;		/* request not ready */

/* Get Host I/F register top address */
	hirp = chbuffp->hif_reg_top;
	ubp_back = ynb_get_ubp( hirp );

/* Write Parameter */
	ynb_set_ubp( hirp, chbuffp->pars_data+2);	/* Set t_mcyc address */
	ynb_set_ubd( hirp, t_mcyc);

/* Write  Com. Par. Request */
	ynb_set_cmd( hirp, YN_CMD_PAR_R );
	ynb_set_ubp( hirp, ubp_back );	/* Set old UBP */

/* Wait Request End */
	data = ynb_get_ints( hirp );
	if ((data & YN_INTS_PAR_I) != 0)
	{
	/* Clear Interrupt Status */
		ynb_clear_ints( hirp, YN_INTS_PAR_I );

	/* Check Error Status */
		data = ynb_get_ests( hirp );
		if ((data & YN_ESTS_PAR_ERR) != 0)
		{
		/* Clear Error Status */
			ynb_clear_ests( hirp, YN_ESTS_PAR_ERR );
			ret = ERROR_INVALID_PARAMETER;
		}
		else
		{
			ret = OK;
		}
	}

	return ret;
}
